<?xml version='1.0' encoding='iso-8859-1'?><TestLog><TestLogDescriptor><TestDescriptor TestName="{{name}}" TestUUID="{{uuid}}"><GeneralInfo><CreationDate>{{creation_date}}</CreationDate><Author>{{author}}</Author><LongDescription>{{description}}</LongDescription></GeneralInfo><ExecutionInformation><Status>{{status}}</Status><LaunchedDate>{{launched}}</LaunchedDate><TerminatedDate>{{terminated}}</TerminatedDate></ExecutionInformation></TestDescriptor></TestLogDescriptor><Events>{% for event in event_list %}<EventDescr Device="{{event['Device']}}" EventID="{{event['id']}}" EventType="{{event['EventType']}}"><Common><EventDate>{{event['EventDate']}}</EventDate><EventTime>{{event['EventTime']}}</EventTime><TestUUID>{{uuid}}</TestUUID></Common><Content><Name>{{event['Name']}}</Name><Category>{{event['Category']}}</Category><Collection>IDB</Collection><Data>{{event['Data']}}</Data></Content>{% if event['EventType'] == 'TC_FEEDBACK' %}<FeedBack><Status>{{event['Status']}}</Status></FeedBack>{% endif %}</EventDescr>{% endfor %}</Events></TestLog>
